@@ -1,9 +1,8 @@
 //===- CodegenCleanup.cpp -------------------------------------------------===//
 //
-//                     The LLVM Compiler Infrastructure
-//
-// This file is distributed under the University of Illinois Open Source
-// License. See LICENSE.TXT for details.
+// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
+// See https://llvm.org/LICENSE.txt for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
 //===----------------------------------------------------------------------===//
 
@@ -13,11 +12,14 @@
 #include "llvm/Analysis/TypeBasedAliasAnalysis.h"
 #include "llvm/IR/Function.h"
 #include "llvm/IR/LegacyPassManager.h"
-#include "llvm/PassInfo.h"
-#include "llvm/PassRegistry.h"
-#include "llvm/PassSupport.h"
+#include "llvm/Pass.h"
 #include "llvm/Support/Debug.h"
+#include "llvm/Transforms/InstCombine/InstCombine.h"
 #include "llvm/Transforms/Scalar.h"
+#include "llvm/Transforms/Scalar/GVN.h"
+#include "llvm/Transforms/Scalar/SimpleLoopUnswitch.h"
+#include "llvm/Transforms/Utils.h"
+
 #define DEBUG_TYPE "polly-cleanup"
 
 using namespace llvm;
@@ -25,7 +27,7 @@ using namespace polly;
 
 namespace {
 
-class CodegenCleanup : public FunctionPass {
+class CodegenCleanup final : public FunctionPass {
 private:
   CodegenCleanup(const CodegenCleanup &) = delete;
   const CodegenCleanup &operator=(const CodegenCleanup &) = delete;
@@ -38,9 +40,9 @@ public:
 
   /// @name FunctionPass interface
   //@{
-  virtual void getAnalysisUsage(llvm::AnalysisUsage &AU) const override {}
+  void getAnalysisUsage(llvm::AnalysisUsage &AU) const override {}
 
-  virtual bool doInitialization(Module &M) override {
+  bool doInitialization(Module &M) override {
     assert(!FPM);
 
     FPM = new llvm::legacy::FunctionPassManager(&M);
@@ -60,40 +62,51 @@ public:
     FPM->add(createCFGSimplificationPass());
     FPM->add(createSROAPass());
     FPM->add(createEarlyCSEPass());
-    FPM->add(createInstructionCombiningPass());
+
+    FPM->add(createPromoteMemoryToRegisterPass());
+    FPM->add(createInstructionCombiningPass(true));
+    FPM->add(createCFGSimplificationPass());
+    FPM->add(createSROAPass());
+    FPM->add(createEarlyCSEPass(true));
+    FPM->add(createSpeculativeExecutionIfHasBranchDivergencePass());
     FPM->add(createJumpThreadingPass());
     FPM->add(createCorrelatedValuePropagationPass());
     FPM->add(createCFGSimplificationPass());
-    FPM->add(createInstructionCombiningPass());
+    FPM->add(createInstructionCombiningPass(true));
+    FPM->add(createLibCallsShrinkWrapPass());
+    FPM->add(createTailCallEliminationPass());
     FPM->add(createCFGSimplificationPass());
     FPM->add(createReassociatePass());
-    FPM->add(createLoopRotatePass());
+    FPM->add(createLoopRotatePass(-1));
+    FPM->add(createGVNPass());
     FPM->add(createLICMPass());
-    FPM->add(createLoopUnswitchPass());
+    FPM->add(createSimpleLoopUnswitchLegacyPass());
     FPM->add(createCFGSimplificationPass());
-    FPM->add(createInstructionCombiningPass());
+    FPM->add(createInstructionCombiningPass(true));
     FPM->add(createIndVarSimplifyPass());
     FPM->add(createLoopIdiomPass());
     FPM->add(createLoopDeletionPass());
     FPM->add(createCFGSimplificationPass());
-    FPM->add(createSimpleLoopUnrollPass());
+    FPM->add(createSimpleLoopUnrollPass(3));
     FPM->add(createMergedLoadStoreMotionPass());
+    FPM->add(createGVNPass());
     FPM->add(createMemCpyOptPass());
+    FPM->add(createSCCPPass());
     FPM->add(createBitTrackingDCEPass());
-    FPM->add(createInstructionCombiningPass());
+    FPM->add(createInstructionCombiningPass(true));
     FPM->add(createJumpThreadingPass());
     FPM->add(createCorrelatedValuePropagationPass());
     FPM->add(createDeadStoreEliminationPass());
     FPM->add(createLICMPass());
-    FPM->add(createLoopRerollPass());
     FPM->add(createAggressiveDCEPass());
     FPM->add(createCFGSimplificationPass());
-    FPM->add(createInstructionCombiningPass());
+    FPM->add(createInstructionCombiningPass(true));
+    FPM->add(createFloat2IntPass());
 
     return FPM->doInitialization();
   }
 
-  virtual bool doFinalization(Module &M) override {
+  bool doFinalization(Module &M) override {
     bool Result = FPM->doFinalization();
 
     delete FPM;
@@ -102,14 +115,15 @@ public:
     return Result;
   }
 
-  virtual bool runOnFunction(llvm::Function &F) override {
+  bool runOnFunction(llvm::Function &F) override {
     if (!F.hasFnAttribute("polly-optimized")) {
-      DEBUG(dbgs() << F.getName()
-                   << ": Skipping cleanup because Polly did not optimize it.");
+      LLVM_DEBUG(
+          dbgs() << F.getName()
+                 << ": Skipping cleanup because Polly did not optimize it.");
       return false;
     }
 
-    DEBUG(dbgs() << F.getName() << ": Running codegen cleanup...");
+    LLVM_DEBUG(dbgs() << F.getName() << ": Running codegen cleanup...");
     return FPM->run(F);
   }
   //@}
