@@ -16,6 +16,7 @@
 #include "polly/Options.h"
 #include "polly/ScopInfo.h"
 #include "polly/ScopPass.h"
+#include "polly/Support/ISLTools.h"
 #include "polly/Support/ScopLocation.h"
 #include "llvm/ADT/Statistic.h"
 #include "llvm/IR/Module.h"
@@ -50,7 +51,8 @@ static cl::opt<std::string>
                   cl::Hidden, cl::value_desc("File postfix"), cl::ValueRequired,
                   cl::init(""), cl::cat(PollyCategory));
 
-struct JSONExporter : public ScopPass {
+class JSONExporter : public ScopPass {
+public:
   static char ID;
   explicit JSONExporter() : ScopPass(ID) {}
 
@@ -64,7 +66,8 @@ struct JSONExporter : public ScopPass {
   void getAnalysisUsage(AnalysisUsage &AU) const override;
 };
 
-struct JSONImporter : public ScopPass {
+class JSONImporter : public ScopPass {
+public:
   static char ID;
   std::vector<std::string> NewAccessStrings;
   explicit JSONImporter() : ScopPass(ID) {}
@@ -215,8 +218,8 @@ static bool importContext(Scop &S, const json::Object &JScop) {
     return false;
   }
 
-  isl::set NewContext = isl::set{S.getIslCtx().get(),
-                                 JScop.getString("context").getValue().str()};
+  isl::set NewContext =
+      isl::set{S.getIslCtx().get(), JScop.getString("context").value().str()};
 
   // Check whether the context was parsed successfully.
   if (NewContext.is_null()) {
@@ -230,8 +233,8 @@ static bool importContext(Scop &S, const json::Object &JScop) {
     return false;
   }
 
-  unsigned OldContextDim = OldContext.dim(isl::dim::param);
-  unsigned NewContextDim = NewContext.dim(isl::dim::param);
+  unsigned OldContextDim = unsignedFromIslSize(OldContext.dim(isl::dim::param));
+  unsigned NewContextDim = unsignedFromIslSize(NewContext.dim(isl::dim::param));
 
   // Check if the imported context has the right number of parameters.
   if (OldContextDim != NewContextDim) {
@@ -287,10 +290,10 @@ static bool importSchedule(Scop &S, const json::Object &JScop,
     }
     Optional<StringRef> Schedule =
         statements[Index].getAsObject()->getString("schedule");
-    assert(Schedule.hasValue() &&
+    assert(Schedule.has_value() &&
            "Schedules that contain extension nodes require special handling.");
     isl_map *Map = isl_map_read_from_str(S.getIslCtx().get(),
-                                         Schedule.getValue().str().c_str());
+                                         Schedule.value().str().c_str());
 
     // Check whether the schedule was parsed successfully
     if (!Map) {
@@ -394,7 +397,7 @@ importAccesses(Scop &S, const json::Object &JScop, const DataLayout &DL,
                << StatementIdx << ".\n";
         return false;
       }
-      StringRef Accesses = JsonMemoryAccess->getString("relation").getValue();
+      StringRef Accesses = *JsonMemoryAccess->getString("relation");
       isl_map *NewAccessMap =
           isl_map_read_from_str(S.getIslCtx().get(), Accesses.str().c_str());
 
@@ -448,14 +451,12 @@ importAccesses(Scop &S, const json::Object &JScop, const DataLayout &DL,
         bool SpecialAlignment = true;
         if (LoadInst *LoadI = dyn_cast<LoadInst>(MA->getAccessInstruction())) {
           SpecialAlignment =
-              LoadI->getAlignment() &&
-              DL.getABITypeAlignment(LoadI->getType()) != LoadI->getAlignment();
+              DL.getABITypeAlign(LoadI->getType()) != LoadI->getAlign();
         } else if (StoreInst *StoreI =
                        dyn_cast<StoreInst>(MA->getAccessInstruction())) {
           SpecialAlignment =
-              StoreI->getAlignment() &&
-              DL.getABITypeAlignment(StoreI->getValueOperand()->getType()) !=
-                  StoreI->getAlignment();
+              DL.getABITypeAlign(StoreI->getValueOperand()->getType()) !=
+              StoreI->getAlign();
         }
 
         if (SpecialAlignment) {
@@ -565,7 +566,7 @@ static bool areArraysEqual(ScopArrayInfo *SAI, const json::Object &Array) {
     return false;
   }
 
-  if (SAI->getName() != Array.getString("name").getValue())
+  if (SAI->getName() != *Array.getString("name"))
     return false;
 
   if (SAI->getNumberOfDimensions() != Array.getArray("sizes")->size())
@@ -574,14 +575,14 @@ static bool areArraysEqual(ScopArrayInfo *SAI, const json::Object &Array) {
   for (unsigned i = 1; i < Array.getArray("sizes")->size(); i++) {
     SAI->getDimensionSize(i)->print(RawStringOstream);
     const json::Array &SizesArray = *Array.getArray("sizes");
-    if (RawStringOstream.str() != SizesArray[i].getAsString().getValue())
+    if (RawStringOstream.str() != SizesArray[i].getAsString().value())
       return false;
     Buffer.clear();
   }
 
   // Check if key 'type' differs from the current one or is not valid.
   SAI->getElementType()->print(RawStringOstream);
-  if (RawStringOstream.str() != Array.getString("type").getValue()) {
+  if (RawStringOstream.str() != Array.getString("type").value()) {
     errs() << "Array has not a valid type.\n";
     return false;
   }
@@ -652,7 +653,7 @@ static bool importArrays(Scop &S, const json::Object &JScop) {
   for (; ArrayIdx < Arrays.size(); ArrayIdx++) {
     const json::Object &Array = *Arrays[ArrayIdx].getAsObject();
     auto *ElementType =
-        parseTextType(Array.get("type")->getAsString().getValue().str(),
+        parseTextType(Array.get("type")->getAsString().value().str(),
                       S.getSE()->getContext());
     if (!ElementType) {
       errs() << "Error while parsing element type for new array.\n";
@@ -661,7 +662,7 @@ static bool importArrays(Scop &S, const json::Object &JScop) {
     const json::Array &SizesArray = *Array.getArray("sizes");
     std::vector<unsigned> DimSizes;
     for (unsigned i = 0; i < SizesArray.size(); i++) {
-      auto Size = std::stoi(SizesArray[i].getAsString().getValue().str());
+      auto Size = std::stoi(SizesArray[i].getAsString()->str());
 
       // Check if the size if positive.
       if (Size <= 0) {
@@ -673,10 +674,10 @@ static bool importArrays(Scop &S, const json::Object &JScop) {
     }
 
     auto NewSAI = S.createScopArrayInfo(
-        ElementType, Array.getString("name").getValue().str(), DimSizes);
+        ElementType, Array.getString("name").value().str(), DimSizes);
 
     if (Array.get("allocation")) {
-      NewSAI->setIsOnHeap(Array.getString("allocation").getValue() == "heap");
+      NewSAI->setIsOnHeap(Array.getString("allocation").value() == "heap");
     }
   }
 
@@ -832,3 +833,49 @@ INITIALIZE_PASS_END(JSONImporter, "polly-import-jscop",
                     "Polly - Import Scops from JSON"
                     " (Reads a .jscop file for each Scop)",
                     false, false)
+
+//===----------------------------------------------------------------------===//
+
+namespace {
+/// Print result from JSONImporter.
+class JSONImporterPrinterLegacyPass final : public ScopPass {
+public:
+  static char ID;
+
+  JSONImporterPrinterLegacyPass() : JSONImporterPrinterLegacyPass(outs()){};
+  explicit JSONImporterPrinterLegacyPass(llvm::raw_ostream &OS)
+      : ScopPass(ID), OS(OS) {}
+
+  bool runOnScop(Scop &S) override {
+    JSONImporter &P = getAnalysis<JSONImporter>();
+
+    OS << "Printing analysis '" << P.getPassName() << "' for region: '"
+       << S.getRegion().getNameStr() << "' in function '"
+       << S.getFunction().getName() << "':\n";
+    P.printScop(OS, S);
+
+    return false;
+  }
+
+  void getAnalysisUsage(AnalysisUsage &AU) const override {
+    ScopPass::getAnalysisUsage(AU);
+    AU.addRequired<JSONImporter>();
+    AU.setPreservesAll();
+  }
+
+private:
+  llvm::raw_ostream &OS;
+};
+
+char JSONImporterPrinterLegacyPass::ID = 0;
+} // namespace
+
+Pass *polly::createJSONImporterPrinterLegacyPass(llvm::raw_ostream &OS) {
+  return new JSONImporterPrinterLegacyPass(OS);
+}
+
+INITIALIZE_PASS_BEGIN(JSONImporterPrinterLegacyPass, "polly-print-import-jscop",
+                      "Polly - Print Scop import result", false, false)
+INITIALIZE_PASS_DEPENDENCY(JSONImporter)
+INITIALIZE_PASS_END(JSONImporterPrinterLegacyPass, "polly-print-import-jscop",
+                    "Polly - Print Scop import result", false, false)
