@@ -35,7 +35,7 @@ enum TYPE {
 } // namespace SCEVType
 
 /// The result the validator returns for a SCEV expression.
-class ValidatorResult {
+class ValidatorResult final {
   /// The type of the expression
   SCEVType::TYPE Type;
 
@@ -112,25 +112,13 @@ public:
   }
 };
 
-raw_ostream &operator<<(raw_ostream &OS, class ValidatorResult &VR) {
+raw_ostream &operator<<(raw_ostream &OS, ValidatorResult &VR) {
   VR.print(OS);
   return OS;
 }
 
-bool polly::isConstCall(llvm::CallInst *Call) {
-  if (Call->mayReadOrWriteMemory())
-    return false;
-
-  for (auto &Operand : Call->arg_operands())
-    if (!isa<ConstantInt>(&Operand))
-      return false;
-
-  return true;
-}
-
 /// Check if a SCEV is valid in a SCoP.
-struct SCEVValidator
-    : public SCEVVisitor<SCEVValidator, class ValidatorResult> {
+class SCEVValidator : public SCEVVisitor<SCEVValidator, ValidatorResult> {
 private:
   const Region *R;
   Loop *Scope;
@@ -142,12 +130,12 @@ public:
                 InvariantLoadsSetTy *ILS)
       : R(R), Scope(Scope), SE(SE), ILS(ILS) {}
 
-  class ValidatorResult visitConstant(const SCEVConstant *Constant) {
+  ValidatorResult visitConstant(const SCEVConstant *Constant) {
     return ValidatorResult(SCEVType::INT);
   }
 
-  class ValidatorResult visitZeroExtendOrTruncateExpr(const SCEV *Expr,
-                                                      const SCEV *Operand) {
+  ValidatorResult visitZeroExtendOrTruncateExpr(const SCEV *Expr,
+                                                const SCEV *Operand) {
     ValidatorResult Op = visit(Operand);
     auto Type = Op.getType();
 
@@ -161,23 +149,23 @@ public:
     return ValidatorResult(SCEVType::PARAM, Expr);
   }
 
-  class ValidatorResult visitPtrToIntExpr(const SCEVPtrToIntExpr *Expr) {
+  ValidatorResult visitPtrToIntExpr(const SCEVPtrToIntExpr *Expr) {
     return visit(Expr->getOperand());
   }
 
-  class ValidatorResult visitTruncateExpr(const SCEVTruncateExpr *Expr) {
+  ValidatorResult visitTruncateExpr(const SCEVTruncateExpr *Expr) {
     return visitZeroExtendOrTruncateExpr(Expr, Expr->getOperand());
   }
 
-  class ValidatorResult visitZeroExtendExpr(const SCEVZeroExtendExpr *Expr) {
+  ValidatorResult visitZeroExtendExpr(const SCEVZeroExtendExpr *Expr) {
     return visitZeroExtendOrTruncateExpr(Expr, Expr->getOperand());
   }
 
-  class ValidatorResult visitSignExtendExpr(const SCEVSignExtendExpr *Expr) {
+  ValidatorResult visitSignExtendExpr(const SCEVSignExtendExpr *Expr) {
     return visit(Expr->getOperand());
   }
 
-  class ValidatorResult visitAddExpr(const SCEVAddExpr *Expr) {
+  ValidatorResult visitAddExpr(const SCEVAddExpr *Expr) {
     ValidatorResult Return(SCEVType::INT);
 
     for (int i = 0, e = Expr->getNumOperands(); i < e; ++i) {
@@ -192,7 +180,7 @@ public:
     return Return;
   }
 
-  class ValidatorResult visitMulExpr(const SCEVMulExpr *Expr) {
+  ValidatorResult visitMulExpr(const SCEVMulExpr *Expr) {
     ValidatorResult Return(SCEVType::INT);
 
     bool HasMultipleParams = false;
@@ -228,7 +216,7 @@ public:
     return Return;
   }
 
-  class ValidatorResult visitAddRecExpr(const SCEVAddRecExpr *Expr) {
+  ValidatorResult visitAddRecExpr(const SCEVAddRecExpr *Expr) {
     if (!Expr->isAffine()) {
       LLVM_DEBUG(dbgs() << "INVALID: AddRec is not affine");
       return ValidatorResult(SCEVType::INVALID);
@@ -283,7 +271,7 @@ public:
     return ZeroStartResult;
   }
 
-  class ValidatorResult visitSMaxExpr(const SCEVSMaxExpr *Expr) {
+  ValidatorResult visitSMaxExpr(const SCEVSMaxExpr *Expr) {
     ValidatorResult Return(SCEVType::INT);
 
     for (int i = 0, e = Expr->getNumOperands(); i < e; ++i) {
@@ -298,7 +286,7 @@ public:
     return Return;
   }
 
-  class ValidatorResult visitSMinExpr(const SCEVSMinExpr *Expr) {
+  ValidatorResult visitSMinExpr(const SCEVSMinExpr *Expr) {
     ValidatorResult Return(SCEVType::INT);
 
     for (int i = 0, e = Expr->getNumOperands(); i < e; ++i) {
@@ -313,7 +301,7 @@ public:
     return Return;
   }
 
-  class ValidatorResult visitUMaxExpr(const SCEVUMaxExpr *Expr) {
+  ValidatorResult visitUMaxExpr(const SCEVUMaxExpr *Expr) {
     // We do not support unsigned max operations. If 'Expr' is constant during
     // Scop execution we treat this as a parameter, otherwise we bail out.
     for (int i = 0, e = Expr->getNumOperands(); i < e; ++i) {
@@ -328,7 +316,7 @@ public:
     return ValidatorResult(SCEVType::PARAM, Expr);
   }
 
-  class ValidatorResult visitUMinExpr(const SCEVUMinExpr *Expr) {
+  ValidatorResult visitUMinExpr(const SCEVUMinExpr *Expr) {
     // We do not support unsigned min operations. If 'Expr' is constant during
     // Scop execution we treat this as a parameter, otherwise we bail out.
     for (int i = 0, e = Expr->getNumOperands(); i < e; ++i) {
@@ -343,6 +331,23 @@ public:
     return ValidatorResult(SCEVType::PARAM, Expr);
   }
 
+  ValidatorResult visitSequentialUMinExpr(const SCEVSequentialUMinExpr *Expr) {
+    // We do not support unsigned min operations. If 'Expr' is constant during
+    // Scop execution we treat this as a parameter, otherwise we bail out.
+    for (int i = 0, e = Expr->getNumOperands(); i < e; ++i) {
+      ValidatorResult Op = visit(Expr->getOperand(i));
+
+      if (!Op.isConstant()) {
+        LLVM_DEBUG(
+            dbgs()
+            << "INVALID: SCEVSequentialUMinExpr has a non-constant operand");
+        return ValidatorResult(SCEVType::INVALID);
+      }
+    }
+
+    return ValidatorResult(SCEVType::PARAM, Expr);
+  }
+
   ValidatorResult visitGenericInst(Instruction *I, const SCEV *S) {
     if (R->contains(I)) {
       LLVM_DEBUG(dbgs() << "INVALID: UnknownExpr references an instruction "
@@ -353,18 +358,6 @@ public:
     return ValidatorResult(SCEVType::PARAM, S);
   }
 
-  ValidatorResult visitCallInstruction(Instruction *I, const SCEV *S) {
-    assert(I->getOpcode() == Instruction::Call && "Call instruction expected");
-
-    if (R->contains(I)) {
-      auto Call = cast<CallInst>(I);
-
-      if (!isConstCall(Call))
-        return ValidatorResult(SCEVType::INVALID, S);
-    }
-    return ValidatorResult(SCEVType::PARAM, S);
-  }
-
   ValidatorResult visitLoadInstruction(Instruction *I, const SCEV *S) {
     if (R->contains(I) && ILS) {
       ILS->insert(cast<LoadInst>(I));
@@ -454,8 +447,6 @@ public:
         return visitSDivInstruction(I, Expr);
       case Instruction::SRem:
         return visitSRemInstruction(I, Expr);
-      case Instruction::Call:
-        return visitCallInstruction(I, Expr);
       default:
         return visitGenericInst(I, Expr);
       }
@@ -470,36 +461,8 @@ public:
   }
 };
 
-class SCEVHasIVParams {
-  bool HasIVParams = false;
-
-public:
-  SCEVHasIVParams() {}
-
-  bool follow(const SCEV *S) {
-    const SCEVUnknown *Unknown = dyn_cast<SCEVUnknown>(S);
-    if (!Unknown)
-      return true;
-
-    CallInst *Call = dyn_cast<CallInst>(Unknown->getValue());
-
-    if (!Call)
-      return true;
-
-    if (isConstCall(Call)) {
-      HasIVParams = true;
-      return false;
-    }
-
-    return true;
-  }
-
-  bool isDone() { return HasIVParams; }
-  bool hasIVParams() { return HasIVParams; }
-};
-
 /// Check whether a SCEV refers to an SSA name defined inside a region.
-class SCEVInRegionDependences {
+class SCEVInRegionDependences final {
   const Region *R;
   Loop *Scope;
   const InvariantLoadsSetTy &ILS;
@@ -515,11 +478,6 @@ public:
     if (auto Unknown = dyn_cast<SCEVUnknown>(S)) {
       Instruction *Inst = dyn_cast<Instruction>(Unknown->getValue());
 
-      CallInst *Call = dyn_cast<CallInst>(Unknown->getValue());
-
-      if (Call && isConstCall(Call))
-        return false;
-
       if (Inst) {
         // When we invariant load hoist a load, we first make sure that there
         // can be no dependences created by it in the Scop region. So, we should
@@ -530,7 +488,7 @@ public:
         // are strictly not necessary by tracking the invariant load as a
         // scalar.
         LoadInst *LI = dyn_cast<LoadInst>(Inst);
-        if (LI && ILS.count(LI) > 0)
+        if (LI && ILS.contains(LI))
           return false;
       }
 
@@ -559,9 +517,8 @@ public:
   bool hasDependences() { return HasInRegionDeps; }
 };
 
-namespace polly {
 /// Find all loops referenced in SCEVAddRecExprs.
-class SCEVFindLoops {
+class SCEVFindLoops final {
   SetVector<const Loop *> &Loops;
 
 public:
@@ -575,14 +532,14 @@ public:
   bool isDone() { return false; }
 };
 
-void findLoops(const SCEV *Expr, SetVector<const Loop *> &Loops) {
+void polly::findLoops(const SCEV *Expr, SetVector<const Loop *> &Loops) {
   SCEVFindLoops FindLoops(Loops);
   SCEVTraversal<SCEVFindLoops> ST(FindLoops);
   ST.visitAll(Expr);
 }
 
 /// Find all values referenced in SCEVUnknowns.
-class SCEVFindValues {
+class SCEVFindValues final {
   ScalarEvolution &SE;
   SetVector<Value *> &Values;
 
@@ -616,31 +573,24 @@ public:
   bool isDone() { return false; }
 };
 
-void findValues(const SCEV *Expr, ScalarEvolution &SE,
-                SetVector<Value *> &Values) {
+void polly::findValues(const SCEV *Expr, ScalarEvolution &SE,
+                       SetVector<Value *> &Values) {
   SCEVFindValues FindValues(SE, Values);
   SCEVTraversal<SCEVFindValues> ST(FindValues);
   ST.visitAll(Expr);
 }
 
-bool hasIVParams(const SCEV *Expr) {
-  SCEVHasIVParams HasIVParams;
-  SCEVTraversal<SCEVHasIVParams> ST(HasIVParams);
-  ST.visitAll(Expr);
-  return HasIVParams.hasIVParams();
-}
-
-bool hasScalarDepsInsideRegion(const SCEV *Expr, const Region *R,
-                               llvm::Loop *Scope, bool AllowLoops,
-                               const InvariantLoadsSetTy &ILS) {
+bool polly::hasScalarDepsInsideRegion(const SCEV *Expr, const Region *R,
+                                      llvm::Loop *Scope, bool AllowLoops,
+                                      const InvariantLoadsSetTy &ILS) {
   SCEVInRegionDependences InRegionDeps(R, Scope, AllowLoops, ILS);
   SCEVTraversal<SCEVInRegionDependences> ST(InRegionDeps);
   ST.visitAll(Expr);
   return InRegionDeps.hasDependences();
 }
 
-bool isAffineExpr(const Region *R, llvm::Loop *Scope, const SCEV *Expr,
-                  ScalarEvolution &SE, InvariantLoadsSetTy *ILS) {
+bool polly::isAffineExpr(const Region *R, llvm::Loop *Scope, const SCEV *Expr,
+                         ScalarEvolution &SE, InvariantLoadsSetTy *ILS) {
   if (isa<SCEVCouldNotCompute>(Expr))
     return false;
 
@@ -680,9 +630,9 @@ static bool isAffineExpr(Value *V, const Region *R, Loop *Scope,
   return true;
 }
 
-bool isAffineConstraint(Value *V, const Region *R, llvm::Loop *Scope,
-                        ScalarEvolution &SE, ParameterSetTy &Params,
-                        bool OrExpr) {
+bool polly::isAffineConstraint(Value *V, const Region *R, Loop *Scope,
+                               ScalarEvolution &SE, ParameterSetTy &Params,
+                               bool OrExpr) {
   if (auto *ICmp = dyn_cast<ICmpInst>(V)) {
     return isAffineConstraint(ICmp->getOperand(0), R, Scope, SE, Params,
                               true) &&
@@ -700,11 +650,12 @@ bool isAffineConstraint(Value *V, const Region *R, llvm::Loop *Scope,
   if (!OrExpr)
     return false;
 
-  return isAffineExpr(V, R, Scope, SE, Params);
+  return ::isAffineExpr(V, R, Scope, SE, Params);
 }
 
-ParameterSetTy getParamsInAffineExpr(const Region *R, Loop *Scope,
-                                     const SCEV *Expr, ScalarEvolution &SE) {
+ParameterSetTy polly::getParamsInAffineExpr(const Region *R, Loop *Scope,
+                                            const SCEV *Expr,
+                                            ScalarEvolution &SE) {
   if (isa<SCEVCouldNotCompute>(Expr))
     return ParameterSetTy();
 
@@ -717,7 +668,7 @@ ParameterSetTy getParamsInAffineExpr(const Region *R, Loop *Scope,
 }
 
 std::pair<const SCEVConstant *, const SCEV *>
-extractConstantFactor(const SCEV *S, ScalarEvolution &SE) {
+polly::extractConstantFactor(const SCEV *S, ScalarEvolution &SE) {
   auto *ConstPart = cast<SCEVConstant>(SE.getConstant(S->getType(), 1));
 
   if (auto *Constant = dyn_cast<SCEVConstant>(S))
@@ -776,9 +727,9 @@ extractConstantFactor(const SCEV *S, ScalarEvolution &SE) {
   return std::make_pair(ConstPart, SE.getMulExpr(LeftOvers));
 }
 
-const SCEV *tryForwardThroughPHI(const SCEV *Expr, Region &R,
-                                 ScalarEvolution &SE, LoopInfo &LI,
-                                 const DominatorTree &DT) {
+const SCEV *polly::tryForwardThroughPHI(const SCEV *Expr, Region &R,
+                                        ScalarEvolution &SE,
+                                        ScopDetection *SD) {
   if (auto *Unknown = dyn_cast<SCEVUnknown>(Expr)) {
     Value *V = Unknown->getValue();
     auto *PHI = dyn_cast<PHINode>(V);
@@ -789,7 +740,7 @@ const SCEV *tryForwardThroughPHI(const SCEV *Expr, Region &R,
 
     for (unsigned i = 0; i < PHI->getNumIncomingValues(); i++) {
       BasicBlock *Incoming = PHI->getIncomingBlock(i);
-      if (isErrorBlock(*Incoming, R, LI, DT) && R.contains(Incoming))
+      if (SD->isErrorBlock(*Incoming, R) && R.contains(Incoming))
         continue;
       if (Final)
         return Expr;
@@ -802,12 +753,12 @@ const SCEV *tryForwardThroughPHI(const SCEV *Expr, Region &R,
   return Expr;
 }
 
-Value *getUniqueNonErrorValue(PHINode *PHI, Region *R, LoopInfo &LI,
-                              const DominatorTree &DT) {
+Value *polly::getUniqueNonErrorValue(PHINode *PHI, Region *R,
+                                     ScopDetection *SD) {
   Value *V = nullptr;
   for (unsigned i = 0; i < PHI->getNumIncomingValues(); i++) {
     BasicBlock *BB = PHI->getIncomingBlock(i);
-    if (!isErrorBlock(*BB, *R, LI, DT)) {
+    if (!SD->isErrorBlock(*BB, *R)) {
       if (V)
         return nullptr;
       V = PHI->getIncomingValue(i);
@@ -816,4 +767,3 @@ Value *getUniqueNonErrorValue(PHINode *PHI, Region *R, LoopInfo &LI,
 
   return V;
 }
-} // namespace polly
